#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ksfest` package."""


import unittest
import click
from click.testing import CliRunner
#from ksfest import ksfest


class TestKsfest(unittest.TestCase):
    """Tests for `ksfest` package."""


    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    #def test_ks_fest(self):
    #    import pandas as pd
    #    import seaborn as sns

    #    iris = sns.load_dataset('iris')
    #    gks= ksfest.ks_fest()
    #   output= gks.get_ks(iris, var_dim='species', sample=0.3, na_number=-1)
    #   num_cols=output.shape[1]
    #   self.assertEqual(num_cols, len(gks.cols) +1) # assert number of columns


import pandas as pd
import numpy as np


def read_sample(filename, sample_size, verbose =False, **kwargs):

	"""

    Parameters
    ----------
    
    filename: csv file tipe
	sample_size: if value is less than 1 
	verbose: if true, it will print all reading and sampling stages
    kwargs: all other parameters from pandas.read_csv() function

    values
    ------  
     """
	

	n = sum(1 for line in open(filename)) - 1 #number of records in file (excludes header)

	if verbose:
		print((str(n) + ' total lines to read'))

	assert sample_size>0

	if sample_size<1:
		s = int(sample_size*n) #desired sample size
	else:
		s=sample_size

	if verbose:
		print((str(s) + ' samples'))

	skip = np.sort(np.random.choice(range(1, n+1),n-s)) #the 0-indexed header will not be included in the skip list
	
	df = pandas.read_csv(filename, skiprows=skip, nrows=s, **kwargs)







# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# Get the project root dir, which is the parent dir of this
import sys
import os

cwd = os.getcwd()
project_root = os.path.dirname(cwd)
from pathlib import Path

# Insert the project root dir as the first element in the PYTHONPATH.
# This lets us ensure that the source package is imported, and that its
# version is used.


#sys.path.insert(0, project_root)

#sys.path.insert(0, Path(os.path.abspath('..')).parents[0])

#import ksfest

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = {
    '.rst': 'restructuredtext',
    '.md': 'markdown',
}
# The master toctree document.
master_doc = 'index'


# -- Project information -----------------------------------------------------

project = 'ksfest'
copyright = '2019, Oriel Zambrano Vergara'
author = 'Oriel Zambrano Vergara'

# The short X.Y version.
version = "0.1.7"
# The full version, including alpha/beta/rc tags.
release = "0.1.7"



# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["sphinx.ext.autodoc", 
	'sphinx.ext.viewcode',
    'sphinx.ext.intersphinx', 
    'sphinx.ext.autosummary',
    'sphinx.ext.todo',
    'sphinx.ext.todo'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#

html_theme = "sphinx_rtd_theme"	



# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# Output file base name for HTML help builder.
htmlhelp_basename = 'ksfestdoc'




# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'ksfest.tex',
     u'ksfest Documentation',
     u'Oriel Zambrano Vergara', 'manual'),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'ksfest',
     u'ksfest Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'ksfest',
     u'ksfest Documentation',
     author,
     'ksfest',
     'One line description of project.',
     'Miscellaneous'),
]



import itertools
from scipy.stats import ks_2samp
from numba import jit
from tqdm import tqdm
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from .utils import read_sample

class ks_fest(object):

    def __init__(self):

        self.cols=list()
        self.dict_ks=dict()
        self.dict_ks_pvalues=dict()
        self.cols=None

    def get_ks(self,df,var_dim, sample,columns=None, na_number=-1, **kwargs):

        """

        Fit and save cdf to check data quality.

        Parameters
        ----------
        df: pandas dataframe or csv file
        var_dim: string 
        
        sample: samplin portion from dataframe
        columsn: list of columsn to calculate ks
        na_number: default number if data is missing
        Attributes
        ----------
        
        """

        if not isinstance(df, pd.DataFrame):
            df=read_sample(df, sample_size=sample, **kwargs)

        if columns==None:
                columns=[col for col in df.columns if col!=var_dim]
        
        #Assing columns to an object list
        self.cols=columns
        
        #Teste

        if not isinstance(columns,list) and not isinstance(columns, np.ndarray):
            raise ValueError("Columns must be a list or a numpy dataframe os strings") 

        try:
            all(df.dtypes.values==float)
        except:
            raise TypeError("Only numeric columns are allowed")

        for comb in tqdm(itertools.combinations(np.unique(df[var_dim]),2)):
            ks_list=[]
            pvalue_list=[]


            for col in columns:
                ks_result=ks_2samp(df.loc[df[var_dim]==comb[0], col].sample(frac=sample).fillna(na_number), df.loc[df[var_dim]==comb[1], col].sample(frac=sample).fillna(na_number))
                ks_list.append(ks_result[0])
                pvalue_list.append(ks_result[1])

            self.dict_ks[str(comb[0])+'_'+str(comb[1])] = ks_list
            self.dict_ks_pvalues[str(comb[0])+'_'+str(comb[1])] = pvalue_list

        pandas_ks_=pd.DataFrame().from_dict(self.dict_ks)
        self.pandas_ks= pandas_ks_.T
        self.pandas_ks.columns=columns
        self.pandas_ks[var_dim]=self.pandas_ks.index
        self.pandas_ks.index=range(len(self.pandas_ks))
        return self.pandas_ks

    def get_ks_numba(self,df,var_dim, sample,columns=None, na_number=-1, **kwargs):

        """

        Fit and save cdf to check data quality numba

        Parameters
        ----------
        df: pandas dataframe or csv file
        var_dim: string 
        
        sample: samplin portion from dataframe
        columsn: list of columsn to calculate ks
        na_number: default number if data is missing
        Attributes
        ----------
        
        """

        if not isinstance(df, pd.DataFrame):
            df=read_sample(df, sample_size=sample, **kwargs)

        if columns==None:
                columns=[col for col in df.columns if col!=var_dim]
        
        #Assing columns to an object list
        self.cols=columns
        
        #Teste

        if not isinstance(columns,list) and not isinstance(columns, np.ndarray):
            raise ValueError("Columns must be a list or a numpy dataframe os strings") 

        try:
            all(df.dtypes.values==float)
        except:
            raise TypeError("Only numeric columns are allowed")

        @jit(nopython=True)
        for comb in tqdm(itertools.combinations(np.unique(df[var_dim]),2)):
            ks_list=[]
            pvalue_list=[]


            for col in columns:
                ks_result=ks_2samp(df.loc[df[var_dim]==comb[0], col].sample(frac=sample).fillna(na_number), df.loc[df[var_dim]==comb[1], col].sample(frac=sample).fillna(na_number))
                ks_list.append(ks_result[0])
                pvalue_list.append(ks_result[1])

            self.dict_ks[str(comb[0])+'_'+str(comb[1])] = ks_list
            self.dict_ks_pvalues[str(comb[0])+'_'+str(comb[1])] = pvalue_list

        pandas_ks_=pd.DataFrame().from_dict(self.dict_ks)
        self.pandas_ks= pandas_ks_.T
        self.pandas_ks.columns=columns
        self.pandas_ks[var_dim]=self.pandas_ks.index
        self.pandas_ks.index=range(len(self.pandas_ks))
        return self.pandas_ks


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from codecs import open
from os import path
from setuptools import setup, find_packages


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


#with open(path.join('requirements.txt'), encoding='utf-8') as f:
#    requirements = [line.strip() for line in f if line]


setup_requirements = []

test_requirements = []

setup(
    author="Oriel Zambrano Vergara",
    author_email='oristides@gmail.com',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python package to check data quality just using ks tests",
    entry_points={
        'console_scripts': [
            'ksfest=ksfest.cli:main',
        ],
    },
    install_requires=['numpy>=1.15', 'numba', 'pandas>=0.19','scipy>=0.18','tqdm>=4.38.0', 'click>=7.0'],
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='ksfest',
    name='ksfest',
    packages=find_packages(include=['ksfest', 'ksfest.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=['numpy>=1.15','pandas>=0.19','scipy>=0.18','tqdm>=4.38.0','matplotlib==3.1.1', 'click>=7.0'],
    url='https://github.com/oristides/ksfest',
    download_url="https://github.com/oristides/ksfest/archive/0.1.10.tar.gz",
    version='0.1.10',
    zip_safe=False,
)
